//+------------------------------------------------------------------+
//|                        Smoothed Rsi Inverse Fisher Transform.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1  DarkSlateGray
#property indicator_color2  LimeGreen
#property indicator_color3  LimeGreen
#property indicator_color4  Orange
#property indicator_color5  Orange
#property indicator_color6  Gold
#property indicator_width1  2
#property indicator_width2  2
#property indicator_width3  2
#property indicator_width4  2
#property indicator_width5  2
#property indicator_minimum -5
#property indicator_maximum 105

//
//
//
//
//

extern string TimeFrame        = "Current time frame";
extern int    Length           = 14;
extern int    MomLength        = 4;
extern bool   UseFilteredPrice = false;
extern int    EmaPeriod        = 4;
extern int    SignalPeriod     = 5;
extern int    RwmaLength       = 2;
extern int    Depth            = 10;
extern int    Price            = PRICE_CLOSE;
extern double LevelUp          = 88;
extern double LevelDown        = 12;
extern bool   Interpolate      = true;

extern bool   alertsOn        = false;
extern bool   alertsOnCurrent = false;
extern bool   alertsMessage   = true;
extern bool   alertsSound     = false;
extern bool   alertsEmail     = false;

//
//
//
//
//

#define MAX_depth 50
double  inv[];
double  invUa[];
double  invUb[];
double  invDa[];
double  invDb[];
double  trend[];
double  trends[];
double  signal[];
double  pBuffer[][MAX_depth];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
      IndicatorBuffers(8);
      SetIndexBuffer(0,inv);
      SetIndexBuffer(1,invUa);
      SetIndexBuffer(2,invUb);
      SetIndexBuffer(3,invDa);
      SetIndexBuffer(4,invDb);
      SetIndexBuffer(5,signal);
      SetIndexBuffer(6,trend);
      SetIndexBuffer(7,trends);
         SetLevelValue(0,LevelUp);
         SetLevelValue(1,LevelDown);
      
      //
      //
      //
      //
      //
      
      RwmaLength = MathMax(RwmaLength,2);
      Depth      = MathMax(MathMin(Depth,MAX_depth),2);
      
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame=="returnBars";      if (returnBars)      return(0);
      calculateValue    = TimeFrame=="calculateValue";  if (calculateValue)  return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);  
            
      //
      //
      //
      //
      //
        
   IndicatorShortName(timeFrameToString(timeFrame)+ " Rwma Smoothed Rsx Inverse Fisher Transform ("+Length+","+EmaPeriod+","+RwmaLength+","+Depth+")");
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double wrkBuffer[][13];
double work[][2];
#define _ema1 0
#define _ema2 1

//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,l,r,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=Bars-counted_bars;
           if (returnBars) { inv[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   if (calculateValue || timeFrame == Period())
   {
      if (ArrayRange(pBuffer,0) != Bars) { ArrayResize(pBuffer,Bars); ArrayResize(work,Bars); ArrayResize(wrkBuffer,Bars); }
      if (trend[limit]== 1) CleanPoint(limit,invUa,invUb);
      if (trend[limit]==-1) CleanPoint(limit,invDa,invDb);
      double weight = (RwmaLength+1.0)*RwmaLength*0.5;
      
      //
      //
      //
      //
      //
      
      double alpha  = 2.0 / (1.0+EmaPeriod);
      double alphas = 2.0 / (1.0+SignalPeriod);
      double Kg = (3.0)/(2.0+Length); 
      double Hg = 1.0-Kg;
      for(i=limit, r=Bars-i-1; i>=0; i--, r++)
      {
         double price = iMA(NULL,0,1,0,MODE_SMA,Price,i);
      
         //
         //
         //
         //
         //

         double coeff  = 5;
         double sum    = 0;
         double sumc   = 0;
         for (int s=0; s<Depth; s++)
         {
            pBuffer[r][s] = price;
            if (r>RwmaLength)
            {
               for (l=0, price=0; l<RwmaLength; l++) price += (RwmaLength-l)*pBuffer[r-l][s];
                                                     price /= weight;
            }
            sum  += coeff*price;
            sumc += coeff;
            coeff = MathMax(coeff-1,1);
         }
         double rwm = sum/sumc;
   
      //
      //
      //
      //
      //
   
         if (UseFilteredPrice)
               wrkBuffer[r][12] = rwm;
         else  wrkBuffer[r][12] = iMA(NULL,0,1,0,MODE_SMA,Price,i);
         if (i==(Bars-1)) { for (int c=0; c<12; c++) wrkBuffer[r][c] = 0; continue; }  

      //
      //
      //
      //
      //
      
      double mom = wrkBuffer[r][12]-wrkBuffer[r-MomLength][12];
      double moa = MathAbs(mom);
      for (int k=0; k<3; k++)
      {
         int kk = k*2;
            wrkBuffer[r][kk+0] = Kg*mom                + Hg*wrkBuffer[r-1][kk+0];
            wrkBuffer[r][kk+1] = Kg*wrkBuffer[r][kk+0] + Hg*wrkBuffer[r-1][kk+1]; mom = 1.5*wrkBuffer[r][kk+0] - 0.5 * wrkBuffer[r][kk+1];
            wrkBuffer[r][kk+6] = Kg*moa                + Hg*wrkBuffer[r-1][kk+6];
            wrkBuffer[r][kk+7] = Kg*wrkBuffer[r][kk+6] + Hg*wrkBuffer[r-1][kk+7]; moa = 1.5*wrkBuffer[r][kk+6] - 0.5 * wrkBuffer[r][kk+7];
      }
      if (moa != 0)
           double rsx = 0.1 * (MathMax(MathMin((mom/moa+1.0)*50.0,100.00),0.00)-50); 
      else        rsx = 0.1 * ((50.0)-50);
      
      
                work[r][_ema1] = work[r-1][_ema1]+alpha*(rsx           -work[r-1][_ema1]);
                work[r][_ema2] = work[r-1][_ema2]+alpha*(work[r][_ema1]-work[r-1][_ema1]);
         double zlema          = 2.0*work[r][_ema1]-work[r][_ema2];
                inv[i]         = 50.0*(1+(MathExp(2*zlema)-1)/(MathExp(2*zlema)+1));
                signal[i]      = signal[i+1]+alphas*(inv[i]-signal[i+1]);
                invUa[i]       = EMPTY_VALUE;
                invUb[i]       = EMPTY_VALUE;
                invDa[i]       = EMPTY_VALUE;
                invDb[i]       = EMPTY_VALUE;
                trend[i]       = 0;
                trends[i]      = trends[i+1];
 
               if (inv[i] > LevelUp)   trend[i] = 1;
               if (inv[i] < LevelDown) trend[i] =-1;
               if (inv[i] > signal[i]) trends[i]= 1;
               if (inv[i] < signal[i]) trends[i]=-1;
               if (trend[i]== 1) PlotPoint(i,invUa,invUb,inv);
               if (trend[i]==-1) PlotPoint(i,invDa,invDb,inv);
      }      

      //
      //
      //
      //
      //
   
      manageAlerts();      
      return(0);
   }

   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   if (trend[limit]== 1) CleanPoint(limit,invUa,invUb);
   if (trend[limit]==-1) CleanPoint(limit,invDa,invDb);
   for (i=limit;i>=0; i--)
   {
      int y     = iBarShift(NULL,timeFrame,Time[i]);
      inv[i]    = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Length,MomLength,UseFilteredPrice,EmaPeriod,SignalPeriod,RwmaLength,Depth,Price,LevelUp,LevelDown,0,y);
      signal[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Length,MomLength,UseFilteredPrice,EmaPeriod,SignalPeriod,RwmaLength,Depth,Price,LevelUp,LevelDown,5,y);
      invUa[i]  = EMPTY_VALUE;
      invUb[i]  = EMPTY_VALUE;
      invDa[i]  = EMPTY_VALUE;
      invDb[i]  = EMPTY_VALUE;
      trend[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Length,MomLength,UseFilteredPrice,EmaPeriod,SignalPeriod,RwmaLength,Depth,Price,LevelUp,LevelDown,6,y);
      trends[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Length,MomLength,UseFilteredPrice,EmaPeriod,SignalPeriod,RwmaLength,Depth,Price,LevelUp,LevelDown,7,y);
      
      //
      //
      //
      //
      //
   
      if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         datetime time = iTime(NULL,timeFrame,y);
         for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
         double factor = 1.0 / n;
         for(int j = 1; j < n; j++)
         {
     	      inv[i+j]    = j*factor*inv[i+n]    + (1.0-j*factor)*inv[i];
     	      signal[i+j] = j*factor*signal[i+n] + (1.0-j*factor)*signal[i];
         }     	      
   }
   for (i=limit;i>=0;i--)
   {
      if (trend[i]== 1) PlotPoint(i,invUa,invUb,inv);
      if (trend[i]==-1) PlotPoint(i,invDa,invDb,inv);
   }
         

   //
   //
   //
   //
   //
   
   manageAlerts(); 
   return(0);
}

//+------------------------------------------------------------------+
//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));

      //
      //
      //
      //
      //
            
      static datetime time1 = 0;
      static string   mess1 = "";
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(time1,mess1,whichBar,"crossed level up");
         if (trend[whichBar] == -1) doAlert(time1,mess1,whichBar,"crossed level down");
      }
      static datetime time2 = 0;
      static string   mess2 = "";
      if (trends[whichBar] != trends[whichBar+1])
      {
         if (trends[whichBar] ==  1) doAlert(time2,mess2,whichBar,"crossed signal line up");
         if (trends[whichBar] == -1) doAlert(time2,mess2,whichBar,"crossed signal line down");
      }
   }
}

//
//
//
//
//

void doAlert(datetime& previousTime, string& previousAlert, int forBar, string doWhat)
{
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," ",timeFrameToString(timeFrame)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Rwma Rsi Inv Fisher ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Rwma Rsi Inv Fisher "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

//
//
//
//
//

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (first[i+1] == EMPTY_VALUE)
      {
         if (first[i+2] == EMPTY_VALUE) {
                first[i]   = from[i];
                first[i+1] = from[i+1];
                second[i]  = EMPTY_VALUE;
            }
         else {
                second[i]   =  from[i];
                second[i+1] =  from[i+1];
                first[i]    = EMPTY_VALUE;
            }
      }
   else
      {
         first[i]  = from[i];
         second[i] = EMPTY_VALUE;
      }
}
  
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}